#######                                                #########
#######              R-Code zur Dissertation           #########
#######                                                #########

source("http://bioconductor.org/biocLite.R")
biocLite("affy")
library(annotate)
library(topGO)
library(hgu133a.db)
affyLib = "hgu133a.db"
affy = "hgu133a"
# bzw. je nach Afymetrix-Chip 
# library(hgu133plus2.db)
# affyLib = "hgu133plus2.db"
# affy = "hgu133plu2"
library(GOSim)
library(limma)
library(bayesmix)
library(cluster)
library(DPpackage)
library(mcclust)
library(DIRECT)
library(foreign)
library(survival)

####### Die zu analysierenden Brustkrebsdaten einlesen.
# Die zugehrigen Matrizen enthalten log2-transformierten und RMA-normalisierten Expressionswerte.
# Zeilen: Probesets, Spalten: Patientennr.
# z.B. mainz[1:2,1:2]             
#               BC6001r     BC6002r     ...
# 1007_s_at     10.702      11.439      ...
# 1053_at        6.980      7.077       ...
#   ...           ...        ...        ...

####### Die zu analysierenden MCF7-Daten einlesen.
# Die MCF7-Datenmatrix enthlt fr die Probesets (Zeilen) je 18 log2-transformierte Genexpressionswerte: 
# fr jede der drei Triplikate eine Messung zu jeweils 6 Zeitpunkten.               
# Fr alle Verfahren auer DIB-C und DIRECT werden Mediane gebildet.
# z.B. mcf7[1:2,]
#   	            0h	   6h	    12h	    24h	    3d  	  14d
# 1557055_s_at  8.688	 9.563	 9.334   9.111	 9.137	 9.277
# 1568815_a_at	6.072	 7.031	 6.863   6.453	 6.209	 6.058
#      ...       ...    ...     ...     ...     ...     ...

# Beim Einsatz von sim_xx mssen die Probesets in Gennamen umgewandelt werden:
# z.B. rownames(mainz) <- getSYMBOL(rownames(mainz), affy)


#######                                                #########
#######              GO-Gruppenzugehrigkeit           #########
#######                                                #########
geneList <- factor(rep(1, nrow(mcf7)), levels = 0:1)
names(geneList) <- rownames(mcf7)

GOdata_BP <- new("topGOdata", ontology = "BP", allGenes = geneList, annot = annFUN.db, affyLib = affyLib)
GOgenes_BP <- genes(GOdata_BP)
GOdata_CC <- new("topGOdata", ontology = "CC", allGenes = geneList, annot = annFUN.db, affyLib = affyLib)
GOgenes_CC <- genes(GOdata_CC)
GOdata_MF <- new("topGOdata", ontology = "MF", allGenes = geneList, annot = annFUN.db, affyLib = affyLib)
GOgenes_MF <- genes(GOdata_MF)

mcf7_BP <- mcf7[rownames(mcf7) %in% GOgenes_BP, ]
mcf7_CC <- mcf7[rownames(mcf7) %in% GOgenes_CC, ]
mcf7_MF <- mcf7[rownames(mcf7) %in% GOgenes_MF, ]

# Gene-ID's (Entrez-ID oder auch LocusLink-ID gennant) ermitteln
entrez_id_BP <- getEG(rownames(mcf7_BP), affy)      # 1562
entrez_id_BP <- entrez_id_BP[!is.na(entrez_id_BP)]   
# filtering out genes not mapping to the currently set GO category --- ===> list of 1227 genes reduced to 1227

entrez_id_CC <- getEG(rownames(mcf7_CC), affy)      # 1667
entrez_id_CC <- entrez_id_CC[!is.na(entrez_id_CC)]  
# filtering out genes not mapping to the currently set GO category --- ===> list of 1313 genes reduced to 1206

entrez_id_MF <- getEG(rownames(mcf7_MF), affy)      # 1560
entrez_id_MF <- entrez_id_MF[!is.na(entrez_id_MF)]  
# filtering out genes not mapping to the currently set GO category --- ===> list of 1219 genes reduced to 1157
  
# hnlichkeit zwischen den Genlisten berechnen
sim_BP <- getGeneSim(entrez_id_BP, similarityTerm="relevance", similarity="funSimMax", norm=T) 
sim_CC <- getGeneSim(entrez_id_CC, similarityTerm="relevance", similarity="funSimMax", norm=T) 
sim_MF <- getGeneSim(entrez_id_MF, similarityTerm="relevance", similarity="funSimMax", norm=T) 


#######      Nicht-modellbasierte Methoden    #########

#######                                  #########
#######          DIB-C (U. Krahn)        #########
#######    hier etwas bersichtlicher    #########
#######                                  #########

# signale <- MCF7-Daten 
design <- read.table("E:\\Daten\\mDesign.txt", header=T)

# Umsortierung der Spalten
# Spaltennamen
spa <- dimnames(signale)[2]
spa <- cbind( spa[[1]][1],spa[[1]][7],spa[[1]][13], spa[[1]][2],spa[[1]][8],spa[[1]][14], spa[[1]][3],spa[[1]][9],spa[[1]][15],
              spa[[1]][4],spa[[1]][10],spa[[1]][16], spa[[1]][5],spa[[1]][11],spa[[1]][17], spa[[1]][6],spa[[1]][12],spa[[1]][18])
# Zeilennamen
zei <- dimnames(signale) [[1]]
signale <- cbind(signale[,1],signale[,7],signale[,13], signale[,2],signale[,8],signale[,14], signale[,3],signale[,9],signale[,15],
     signale[,4],signale[,10],signale[,16], signale[,5],signale[,11],signale[,17], signale[,6],signale[,12],signale[,18])
colnames(signale)<- spa
rownames(signale)<- zei

dibc <- function(M, mDesign, SignifVec, minCG=3, n1=100, n2=3000)
{  

    leng <- length( SignifVec )                                           # leng <- 4
    len <- leng^2                                                         # len <- 16
    erstesSignif <- rep(1, len)                                           # 0.005 0.005 0.005 0.005 /0.001 0.001 0.001 0.001 /
                                                                          # 0.050 0.050 0.050 0.050 /0.010 0.010 0.010 0.010
    zweitesSignif <- rep(1, len)                                          # 0.005 0.001 0.050 0.010 /0.005 0.001 0.050 0.010 /
                                                                          # 0.005 0.001 0.050 0.010 /0.005 0.001 0.050 0.010
    for(i in 1:leng) {
       for(j in 1:leng) {      
           erstesSignif[ (i-1)*leng + j ] <- SignifVec[i]
           zweitesSignif[ (i-1)*leng + j ] <- SignifVec[j]
       }
    }      
    len <- len + 1                                                          
    G <- probeNum <- nrow(M)                                             
    T <- ZeitNum <- length( mDesign[,2] )                                       
    p <- ncol(M)                                                                
    zeitpunkt <- as.vector( mDesign[[1]] )                               
    wdhNum    <- as.vector( mDesign[[2]] )                                      
    wdhNum.cum <- cumsum(wdhNum)                                                
    allNull <- paste(  paste(rep("N", ZeitNum-1), collapse=""), ",",      # "NNNNN,NNNN"
                       paste(rep("N", ZeitNum-2), collapse=""), sep="" )
    re <- matrix(allNull, probeNum, len)                                  # 2632 x 17 (da 4 krit Werte) mit "NNNNN,NNNN"

func_Fdiff <- function(M, mDesign)
{ # leere Ausgabematrix
  re <- list()
  re$modT  <- matrix(0, G, T-1)                                           # Ausgabematrix modT: Anzhalgene x p-1 Zeiptunkte
  re$dof   <- rep(0, T-1)                                                 # Ausgabevktor  dof: p-1 Zeipunkte x 1

  for(j in 1:(T-1))
  {  begin <- ifelse(j==1, 1, wdhNum.cum[j-1]+1)                          # beginn:1 4 7 ...
     end   <- wdhNum.cum[j+1]                                             # end:   6 9 12...
     Msub <- M[ , c(begin:end)]                                           # Msub Gene x ausgewhlten Spalten 
                                                                          # (Spalten angeordnet: a1,b1,c1,...,a6,b6,c6)
     design.eb <- cbind(time1=c( rep(1,3), rep(0,3)), 
                        time2=c( rep(0,3), rep(1,3)) )                    # j=1, 2,...
     fit <- lmFit(Msub, design=design.eb)
     contrast.matrix <- cbind( "Group2-Group1"=c(-1,1) )
     fit2 <- contrasts.fit(fit, contrast.matrix)
     eb <- eBayes(fit2)
     re$modT[,j] <- eb$t[,1]                                              # Matrix der moderaten t-Statistiken
     re$dof[j] <- eb$df.residual[1] + eb$df.prior                         # 4 + zustzliche FG
   }                                                                              
  colnames(re$modT) <- paste("Fdi", c(1:(T-1)), sep="")
  re
}

Fdiff <- func_Fdiff(M, mDesign)                                           # Liste mit 2632 x 5 und 5 x 1

func_Amat <- function(Ymat, design)
{ Amat <- matrix(0, G, T)                                                 # Median ber die Wiederholungen, d.h. 2632 x 6 Matrix
  for(i in 1:T)
  {
    if(i == 1) {subCol <- c( 1: wdhNum.cum[i])}                           # falls i==1 subCol <- c( 1: 3)
    else {subCol <- c( (wdhNum.cum[i-1]+1) : wdhNum.cum[i]) }             # sonst 4,5,6 oder 7,8,9 oder ...
    Amat[,i] <- apply(Ymat[,subCol], 1, median)
  }
 return(Amat)
}

Amat <- func_Amat(M, mDesign)                                              

 for(i in 2:len)
{                                                                                

# erstrangiger Unterschied                                          
func_pattern_Fdiff  <- function(M, thres)
{
  nRow <- nrow(M$modT)                                                          
  nCol <- ncol(M$modT)                                                          
  Fmat <- matrix("N", nRow, nCol)                                             
  for(i in 1:nCol) {
    kritWert <- qt(1-thres/2, M$dof[i])                                          
    Fmat[M$modT[,i] >   kritWert, i] <- "I"                               
    Fmat[M$modT[,i] <  -kritWert, i] <- "D"
  }
  res <- rep("N", nCol)
  for(i in 1:nRow) {
    res[i] <- paste( Fmat[i,],  collapse="" )                             # fasst Zeilen von Fmat zu einem Eintrag in res zusammen
  }                                                                       # -> Vektor 2632 x 1
  res
}

tmpF <- func_pattern_Fdiff(M=Fdiff,thres=erstesSignif[i-1])

# zweitrangiger Unterschied
func_pattern_Sdiff <- function(M, thres,n1,n2)
{
    G <- nrow(M$modT)                                                          
    T <- ncol(M$modT) + 1                                                      

    SecStat <- matrix(0, G, T-2)                                          # in SecStat Gene x 4 befinden sich die Differenzen
    for(j in 1:(T-2)) {
       SecStat[,j] <- M$modT[,(j+1)] - M$modT[,j]
    }
    Smat <- matrix("N", G, T-2)

func_tdiff_quantile <- function(df1, df2, alpha=thres, n1,n2)
{
    stat <- rep(0, n1)
    for(i in 1:n1) {
        x.rand <- rt(n2, df1)
        y.rand <- rt(n2, df2)
        w <- x.rand-y.rand
        ord <- order(w, decreasing = TRUE)
        w <- w[ord]
        stat[i] <- w[ max(n2*(alpha/2) , 1) ]
        
     }
    re <- median(stat)  
    re
}

     for(i in 1:(T-2)) {
        kritWert <- func_tdiff_quantile(M$dof[i],M$dof[(i+1)], alpha=thres, n1=n1, n2=n2)
        Smat[SecStat[,i] > kritWert, i] <- "V"
        Smat[SecStat[,i] < -kritWert, i] <- "A"

     }
    res <- rep("N",  G)
    for(i in 1:G) {
        res[i] <- paste( Smat[i,],  collapse="" )
    }
    res
}

tmpS <- func_pattern_Sdiff(Fdiff,n1=n1,n2=n2,thres=zweitesSignif[i-1])
re[,i] <- paste(tmpF, ",", tmpS, sep="")

# Aufteilung zu kleiner Cluster
patTab <- tab_pat <- table(re[,i])                                                        
cl <- re[,i]
reassign_minor <- function(cl, patTab, Amat, minCG=5)
{
    num_probe <- nrow(Amat)                                                    
    num_zeit <- ncol(Amat)                                                    
    pat_minor  <- names( patTab[ patTab <  minCG  ] )                    # enthlt die Cluster mit weniger als minCG Mitglieder
    pat_major  <- names( patTab[ patTab >= minCG ] )                     # enthlt die Cluster mit mehr als minCG Mitglieder
    gene_alle   <- c(1:num_probe)
    gene_minor <- gene_alle[ cl %in% pat_minor]                          # Gene, die in Cluster mit weniger als minCG Mitglieder
    gene_major <- setdiff(gene_alle, gene_minor)                         # Gene, die in Cluster mit mehr als minCG Mitglieder

    for(i in gene_minor) {
        len <- length(gene_major)                                               
        maximizer <- 1
        maxSim <- cor( Amat[i,], Amat[gene_major[1], ] )                       
        for(j in 2:len) {                                                       
            curSim <- cor( Amat[i,], Amat[gene_major[j], ] )
            if( curSim > maxSim ) {
                maximizer <- j
                maxSim <- curSim
            }
        }
        cl[i] <- cl[gene_major[maximizer]]
        # update gene_major 
        gene_major <- append(gene_major, i)
    }
    cl
}
 re[,i] <- reassign_minor(cl=re[,i], tab_pat, Amat, minCG=5)
 }
 re 
}
                      
cluster <- dibc(M=signale,mDesign=design,SignifVec=c(0.05, 0.01, 0.001, 0.0001), minCG=5, n1=1000, n2=10000)


             
#######                                  #########
#######          STEM (U. Krahn)         #########
#######    hier etwas bersichtlicher    #########
#######                                  #########

# Zeilennamen
zei <- dimnames(signale) [[1]]

# 1) Zeitreihen beginnen bei Null: normalisierte Daten (0, v_1-v_0, ..., v_n-v_0)
a <- c(1,7,13)
for(i in a){
signale[,i:(i+5)] <- cbind(signale[,i]-signale[,i],signale[,i+1]-signale[,i],signale[,i+2]-signale[,i],signale[,i+3]-signale[,i],
                           signale[,i+4]-signale[,i],signale[,i+5]-signale[,i])
}

# bei Wiederholungen: mitteln per Median fr die Berechnungen nach Normalisierung
signale_umg <- cbind(signale[,1],signale[,7],signale[,13], signale[,2],signale[,8],signale[,14], signale[,3],signale[,9],signale[,15],
                     signale[,4],signale[,10],signale[,16], signale[,5],signale[,11],signale[,17], signale[,6],signale[,12],signale[,18])

G <-nrow(signale) 
sig <- matrix(0, G, 6)
wdhNum_cum <- c(3,6,9,12,15,18) 

for (i in 1:6){
if(i == 1) {subCol <- c( 1: wdhNum_cum[i])}                                 
else {subCol <- c( (wdhNum_cum[i-1]+1) : wdhNum_cum[i]) }     
sig [,i] <- apply(signale_umg[,subCol],1,median)     
}

colnames(sig)<- c("0h","6h","12h","24h","3d","14d")
rownames(sig) <- zei 

# 2)Parameter c kontrolliert, wie stark Genexpression eines Gens zwischen 
# aufeinander folgeneden Zeitpunkten variieren darf

moeglichen_Profile <- function (ce)
{ 
  moglP <- matrix(0, (2*ce+1)^5, 6)  
  a <- c(ce:(-ce))
  b <- c(1:length(a))
  for (j in 1:5){ 
  d <- 0
  for (i in b){
  d <-  c(d, rep(a[i],((((2*ce+1)^5)/(length(a)^j)))))
  }
  d <- rep(d[-1],(length(a))^(j-1))
  moglP[,j+1]<- d
  }
  moglP
}

var <- function (ce,em){

moglP <- moeglichen_Profile(ce=2)                                         # c=2
moglP[,3] <- (moglP[,2]+moglP[,3]) 
moglP[,4] <- (moglP[,3]+moglP[,4])
moglP[,5] <- (moglP[,4]+moglP[,5])
moglP[,6] <- (moglP[,5]+moglP[,6])

# Das Profil konstant 0 wird ausgeschlossen
p0 <- paste(c(0,0,0,0,0,0),collapse="")
e <- apply(moglP, 1,function(x) paste(x,collapse=""))
stelle <- which(p0==e)
moglP <- moglP[-stelle,]

# 3)Auswahl m potenzieller Profile R, so dass 
# der minimale Abstand zwischen allen Profilen in R maximiert wird

# Matrix der gesuchten Profile mit Anfangsprofil
R <- rbind(c(0,-1,-2,-3,-4,-5))

p1 <- paste(R,collapse="")
e <- apply(moglP, 1,function(x) paste(x,collapse=""))

stelle <- which(p1==e)
# noch mglichen Profile
L <- moglP[-stelle,] 

for (i in 2 : em){
# whle fr jedes p aus L das p1 aus R fr das minimal
minvec <- rep(0,nrow(L))
for (j in 1:nrow(L)){
   maxcor_r<- (cor(L[j,],R[1,]))   
   if(nrow(R) > 1){
   for (k in 2:nrow(R)){ 
    corella <- (cor(L[j,],R[k,])) 
    if(corella > maxcor_r) {
    maxcor_r <- corella 
   }    
   }
   }
 minvec[j]<- maxcor_r
}
  # Stelle in L
  ausgesuchte_profile <- which.min(minvec) 
  # R updaten
  R <- rbind(R,L[ausgesuchte_profile,])
  # L updaten
  L <- L[-ausgesuchte_profile,] 
}

# 4a)Zuordnung der Gene zu den Profilen 
beobAnzahl <- function(sig,R){ 
 cormatrix <- matrix(-9,nrow(sig),nrow(R))
 for (i in 1:nrow(sig)){
   maxcor <- cor(sig[i,],R[1,])
   cormatrix[i,1] <- maxcor 
   for (j in 2:nrow(R)){
   corella <- cor(sig[i,],R[j,])
   cormatrix[i,j] <- corella 
   }
 }
 
 # Gene knnen auch zu mehreren profilen zugeordnet werden!
 welcher <- function(x){
 c(which(x==max(x))) 
 }
 
 # Vektor oder Liste mit den zugeordneten Modellprofil-Nummern
 Zuordnung <- apply(cormatrix,1,welcher)
 
 # 4b)und beobachtete Anzahl pro Profil berechnen
 # Bei jedem Profil Beobachtete Anzahl Gene bestimmen, dabei Gewichtung mit 1/h falls Bindungen
 Profile <- unlist(Zuordnung)
 Gewichte <- unlist(lapply(Zuordnung,length))
 
 vec <- -9
 for (i in 1: length(Gewichte)){
 wieoft <- Gewichte[i]
 was <- 1/(Gewichte[i])
 vec <- append(vec,rep(was,wieoft))
 }
 Gewichte <- vec[-1]
 profgew <- cbind(Profile,Gewichte)
 
 beobAnzahl <- rep(0,nrow(R))
 for (i in 1: nrow(R)){
   a <- 0
   for (j in 1:nrow(profgew)){
     if (profgew[j,1]==i)
     {a <- cbind(a,profgew[j,2])}
   }
   beobAnzahl[i] <- sum(a)   
 }
 list(beobAnzahl=beobAnzahl,Zuordnung=Zuordnung)
 }
beobachtet <- beobAnzahl(sig=sig,R=R) 

# 5a)erwartete Anzahl bestimmen
library(combinat)

# aus allen Permutationen fr jedes Gen, l=m auswhlen
auswahlPerm <- sig[0,]
for (i in 1:nrow(sig)){
allePerm <- t(array(unlist(permn(sig[i,])), dim = c(6, gamma(7))))
auswahlPerm <- rbind(auswahlPerm,allePerm[sample(1:720,50),])
}

# erwartete Anzahl bestimmen
erwAnzahl <- beobAnzahl(sig=auswahlPerm,R=R)

# durch Anzahl Permutationen/Gen (720)
# e <- erwAnzahl$beobAnzahl/(nrow(auswahlPerm)/nrow(sig))
e <- erwAnzahl$beobAnzahl/50
p <- as.matrix(e/nrow(sig))

# 5b)signifikante Profile bestimmen
# X= Anzahl Gene bei jedem Modellprofil ist binom-vtlt mit n=Anzahl Gene, p=e/n
# einseitgier Binomialtest

# 1-alpha-Quantil der Binomvtl. bestimmen mit Bonferroni-Korrektur
alpha <- 0.05/nrow(R)
erw <- qbinom((1-alpha), size=nrow(sig), prob=p[1:nrow(p)])
signifProf <- which (beobachtet$beobAnzahl > erw)
ergebnis <- list(R=R,beobAnzahl=beobachtet,erwAnzahl=erwAnzahl,signifProf=signifProf)
}

var(ce=2,em=50)



#######                                  #########
#######                PFP               #########
#######                                  #########

# Das zugehrige PFP-Tool kann nach Rcksprache mit Autoren zur Verfgung gestellt werden.




#######       Modellbasierte Methoden    #########

# Das Vorgehen ist bei jeder dieser Methoden hnlich: 
# PSM mit comp.psm(Matrix mit MCMC-Samples von Clusterungen) 
# berechnen und anschlieend mit maxpear(psm) und minbinder(psm) 
# Clusterungen optimieren. 

#######                                  #########
#######        Finite Mixture Models     #########
#######                                  #########
# mcf7 <- MSCF7-Daten
prcs <- prcomp(mcf7)
mcf7.1 <-  as.matrix(prcs$x[,1])  # PCA

# alle Parametereinstellungen sind variabel und vom Benutzer vorzugeben:
model_ohneGO <- BMMmodel(mcf7.1, k = 10, priors = list(kind = "independence", parameter = "priorsRaftery", hierarchical = "tau"))
control <- JAGScontrol(variables = c("mu", "tau", "eta", "S"), burn.in = 1000, n.iter = 500, seed = 10)
z <- JAGSrun(mcf7.1, model = model_ohneGO, control = control) 
# z$results[,1:length(rownames(mcf7.1))] :  Matrix mit MCMC-Samples von Clusterungen

# Falls nach GO: analog, nur Datenstze nach GO-Gruppen verwenden.

# Falls die hnlichkeitsmatrix bercksichtigt werden soll: sim_xx (s. oben) fr xx (BP, MF, CC) laden.

# Die hnlichkeitsmatrix in die Optimierungsmethoden von Arno Fritsch einsetzen: 
# mit as.dist(1-sim_xx) anstelle von as.dist(1-PSM).



#######                                  #########
#######              k-means             #########
#######                                  #########

# Fehlerquadratsumme                            
wss <- numeric()
for(k in 2:15)         
{
  wss[k] <- sum(kmeans(mcf7, k, nstart = 20)$ withinss)
}
plot(1:15, wss, col = "blue", xlab = "Anzahl Cluster k", ylab = "Fehlerquadratsumme", type = "b", cex = 0.5,
ylim = c(5000,29000), axes = F)
axis(1, at=seq(2, 15, 1))
axis(2, at=seq(5000,29000, 4000))

# Silhouettenwerte
dissE <- (daisy(mcf7))^2
sil <- numeric()
for(k in 2:15) 
{
  sil[k] <- mean(silhouette(kmeans(mcf7, k, nstart = 20)$cl, dissE)[,3])
}
plot(1:15, sil, col = "blue", xlab = "Anzahl Cluster k", ylab = "average silhouette width", type = "b", 
ylim = c(0.2, 0.8), xlim = c(2,15), cex = 0.5, axes = F)
axis(1, at=seq(2, 15, 1))
axis(2, at=seq(0.2, 0.8, 0.1))

# Falls nach GO: analog, nur Datenstze nach GO-Gruppen verwenden.




#######                                      #########
#######   Dirichlet Process Mixture Models   #########
#######                                      #########

mcf7.2 <-  as.matrix(prcs$x[,1:2])  # PCA

# Funktion von A. Fritsch
get.zdraw <- function(dpdens,n,d)
   {
    randsave <- dpdens$save.state$randsave
    mus <- grep(colnames(randsave),pattern="mu")
    mu1s <- randsave[,mus[(0:(n-1)*d)+1]]
    zdraw <- t(apply(mu1s,1,function(x) floor(rank(x))))
    colnames(zdraw) <- NULL
    nclus2 <- apply(zdraw,1,function(x) length(unique(x)))
    # nclus frs berprfen der Richtigkeit
    nclus <- dpdens$save.state$thetasave[,"ncluster"]
    if(any(nclus2!=nclus)) return("Fehler!")
    zdraw
    }
# alle Parametereinstellungen sind variabel und vom Benutzer vorzugeben:   
dpdens <- DPdensity(y=mcf7.2, prior=list(a0=4, b0=2, k0=1, m1=c(0,0), nu1=5, psiinv1=diag(apply(mcf7.2,2,var))),
                    mcmc=list(nburn=1000, nskip=50, nsave=500, ndisplay=100), state=NULL,status=TRUE)
zdraw <- get.zdraw(dpdens, n=length(rownames(mcf7)), d=2)
# zdraw: Matrix mit MCMC-Samples von Clusterungen

# Falls nach GO: analog, nur Datenstze nach GO-Gruppen verwenden.

# Falls die hnlichkeitsmatrix bercksichtigt werden soll: sim_xx (s. oben) fr xx (BP, MF, CC) laden.

# Die hnlichkeitsmatrix in die Optimierungsmethoden von A. Fritsch einsetzen: 
# analog zu Finite Mixture Models.



#######                               #########
#######              DIRECT           #########
#######                               #########

# ohne GO
# data <- MCF7-Daten

# Falls nach GO: analog, nur Datenstze nach GO-Gruppen verwenden.

# Falls die hnlichkeitsmatrix bercksichtigt werden soll: sim_xx (s. oben) fr xx (BP, MF, CC) laden.
# Die hnlichkeitsmatrix in die Optimierungsmethoden von Arno Fritsch einsetzen: 
# mit as.dist(1-sim_xx) anstelle von as.dist(1-PSM).

# Mediane nur erzeugen, falls Sim bentigt wird;
# falls nur nach GO-Gruppen, Median-Bildung berspringen.
# mediane <- matrix(0, length(rownames(sim_xx)), 18)
# rownames(mediane) <- rownames(sim_xx)
# colnames(mediane) <- colnames(mcf7_xx[1,-c(1:2)])
# for(i in 1:length(rownames(sim_xx))){
# 	tmp.mat <- mcf7_xx[which(mcf7_xx[,2] %in% rownames(sim_xx)[[i]]), -c(1:2), drop=FALSE]
# 	for(j in 1:18){
# 		mediane[i,j] <- median(tmp.mat[,j])
# 	}
# }
# aus Entrez ID's die Gennamen ausgeben lassen 
# rownames(mediane) <- unlist(as.list(org.Hs.egSYMBOL[rownames(mediane)]))
# data <- mediane

# SKIP: Spalten zum berspringen; bei data nachschauen, ob EntrezID in der ersten Spalte und danach Expressionswerte
# - dann SKIP = 1, falls Werte schon in der 1. Spalte - SKIP = 0.
# data.name muss definiert werden, da wir spter die erzeugten Dateien wie mcmc_cs.out u.a. brauchen
# diese liegen dann in diesem Ordner.

# alle Parametereinstellungen sind variabel und vom Benutzer vorzugeben:
DIRECT (data=data, data.name="E:\\Daten\\..." , SKIP=0, nTime=length(times), times=c(0,6,12,24,72,336),
c.curr=rep (1, nGene = nrow(data)), PAR.INIT=TRUE, alpha.curr=0.01, alpha.prior.shape=1/nGene = nrow(data), alpha.prior.rate=1,
nIter=5, burn.in=1000, step.size=1, nResample=2, seed.value=12, RNORM.METHOD="svd", SAMPLE.C="FRBT",
PRIOR.MODEL="OU", ALPHA.METHOD="MH", RELABEL.THRESHOLD=0.01)

tmp.summary = summaryDIRECT (data.name)
# tmp.summary$top.clust.alloc  -->  Clusterung - in Datei(en) abspeichern

# mbind und mpear ohne bzw. mit GO, aber noch ohne Sim_xx:
# davor sollte die *_mcmc_cs.out-Datei (MCMC-Samples) als eine .txt-Datei (z.B. "...ohneGO") abgespeichert werden
datei <- read.table("E:\\Daten\\...ohneGO.txt")
datei <- as.matrix(datei[,-2633])
# datei: Matrix mit MCMC-Samples von Clusterungen

# mit GO, aber noch nicht in die dist  (nur nach Gruppen) analog
# die *_mcmc_cs.out-Dateien je nach GO-Gruppe mit der zugehrigen Endung abspeichern: z.B. "... ohneSimBP"
datei_CC <- read.table("E:\\Daten\\...ohneSimCC.txt")
datei_CC <- as.matrix(datei_CC[,-1668])
datei_BP <- read.table("E:\\Daten\\...ohneSimBP.txt")
datei_BP <- as.matrix(datei_BP[,-1563])
datei_MF <- read.table("E:\\Daten\\...ohneSimMF.txt")
datei_MF <- as.matrix(datei_MF[,-1561])
# datei_xx: Matrizen mit MCMC-Samples von Clusterungen

# Falls die hnlichkeitsmatrix bercksichtig werden soll: Sim_xx (s. oben) fr xx (BP, MF, CC)
datei_CC <- read.table("E:\\Daten\\...ohneSimCC.txt")
datei_CC <- as.matrix(datei_CC[,-1207])
datei_BP <- read.table("E:\\Daten\\...ohneSimBP.txt")
datei_BP <- as.matrix(datei_BP[,-1228])
datei_MF <- read.table("E:\\Daten\\...ohneSimMF.txt")
datei_MF <- as.matrix(datei_MF[,-1158])
# datei_xx: Matrizen mit MCMC-Samples von Clusterungen

# Die hnlichkeitsmatrix in die Optimierungsmethoden von Arno Fritsch einsetzen: 
# analog zu Finite Mixture Models.



#######                                      #########
#######              Auswertung              #########
#######                                      #########

#######                                                  #########
#######    Metagenberechnung nach Freis et al. (2009)    #########
#######                                                  #########
# cluster_mcf7 <- aus dem 1. Schritt des Verfahren result. Clusterungen.
# In den Zeilen zugehriger Matrizen stehen Gene und in den Spalten Clustermethode.
# Ein Beispiel fr das Verfahren DIB-C:
# 	            ...      U6          U7      ...
# 1553972_a_at	...  NNNIN,NNVA	 NNNIN,NNVA  ...
# 1554020_at	  ...  INNNN,NNNN	 INNNN,ANNN  ...
#    ...        ...      ...        ...      ...

liste_cluster <- vector("list", length(cluster_mcf7))
names(liste_cluster) <- colnames(cluster_mcf7)
for(i in 1:length(cluster_mcf7))
    {
    liste_cluster[[i]] <- unique(cluster_mcf7[,i])
    n <- unlist(liste_cluster[i],use.names=FALSE)
    untercluster <- vector("list", length(n))
    names(untercluster) <- n
    for(j in 1:length(n))
       {
        untercluster[[j]] <- rownames(cluster_mcf7)[which(cluster_mcf7[,i]==n[j])]
       }
    liste_cluster[[i]] <- untercluster
    }
liste <- unlist(liste_cluster, recursive = FALSE)

# Metagene fr die jeweilige Kohorte berechnen, z.B. fr die Gesamtkohorte (766 Patientinnen)
metagene_ges <- matrix(0, length(liste), 766) 
rownames(metagene_ges) <- names(liste)
colnames(metagene_ges) <- colnames(gesamt)
for(i in 1:length(names(liste))){
    tmp.mat <- (gesamt[which(rownames(gesamt) %in% liste[[i]]), ,drop=FALSE]
                -apply(gesamt[which(rownames(gesamt) %in% liste[[i]]), ,drop=FALSE], 1, median))/
                 apply(gesamt[which(rownames(gesamt) %in% liste[[i]]), ,drop=FALSE], 1, mad)
	for(j in 1:766){
		metagene_ges[i,j] <- median(tmp.mat[,j])
	}
}
metagene_ges <- t(metagene_ges)
# NA's lschen
metagene_ges <- metagene_ges[,!is.na(metagene_ges[1,])]
liste <- liste[which(names(liste) %in% colnames(metagene_ges))]
listenlaenge <- unlist(lapply(liste,length))
groesser4 <- names(listenlaenge)[which(as.vector(listenlaenge) > 4)]
liste <- liste[which(names(liste) %in% groesser4)]      # nur Cluster mit mehr als 4 probesets
# metagene_ges wieder laufen lassen

# Transbig, Mainz, Rotterdam analog
# --> metagene_tr, metagene_m, metagene_r


#######                                              #########
#######              berlebenszeitanalyse           #########
#######                                              #########

# event und zeit <- berlebenszeitdaten

p <- HR <- xquer <- xmed <- stabw <- matrix(0, length(colnames(metagene_ges)), 1)
for(i in 1:length(colnames(metagene_ges)))
{
	p[i] <- summary(coxph(Surv(zeit,event)~metagene_ges[,i]))[[6]][5]
  HR[i] <- summary(coxph(Surv(zeit,event)~metagene_ges[,i]))[[6]][2]
  xquer[i] <- median(metagene_ges[,i])
  xmed[i] <- mean(metagene_ges[,i])
  stabw[i] <- sd(metagene_ges[,i])
}
pwerte_ges <- cbind(as.numeric(p), as.numeric(HR), as.numeric(xquer), as.numeric(xmed), as.numeric(stabw))
rownames(pwerte_ges) <- colnames(metagene_ges)
colnames(pwerte_ges) <- c("p-value", "HR", "mean", "median", "st. abw.")
pwerte_kleiner0.05_ges <- pwerte_ges[which(pwerte_ges[,1]<=0.050),] 

# Transbig, Mainz, Rotterdam analog


# AAS:
# fr die AAS-Daten analoges Vorgehen mit den entsprechenden AAS-Daten im Ordner "Daten".
